@@ -719,7 +719,12 @@ class Route53Backend(BaseBackend):
             if value["Action"] == "CREATE" and value in the_zone.rr_changes:
                 name = value["ResourceRecordSet"]["Name"] + "."
                 _type = value["ResourceRecordSet"]["Type"]
-                raise ResourceRecordAlreadyExists(name=name, _type=_type)
+                # check if the record exists or just in rr_changes (journal)
+                all_records = list(
+                    the_zone.get_record_sets(start_type=_type, start_name=name)
+                )
+                if all_records:
+                    raise ResourceRecordAlreadyExists(name=name, _type=_type)
 
         for value in change_list:
             if value["Action"] == "DELETE":
