@@ -1,7 +1,7 @@
 from typing import Any, List
 
-from moto.stepfunctions.parser.asl.component.intrinsic.argument.function_argument_list import (
-    FunctionArgumentList,
+from moto.stepfunctions.parser.asl.component.intrinsic.argument.argument import (
+    ArgumentList,
 )
 from moto.stepfunctions.parser.asl.component.intrinsic.function.statesfunction.states_function import (
     StatesFunction,
@@ -16,13 +16,13 @@ from moto.stepfunctions.parser.asl.eval.environment import Environment
 
 
 class Array(StatesFunction):
-    def __init__(self, arg_list: FunctionArgumentList):
+    def __init__(self, argument_list: ArgumentList):
         super().__init__(
             states_name=StatesFunctionName(function_type=StatesFunctionNameType.Array),
-            arg_list=arg_list,
+            argument_list=argument_list,
         )
 
     def _eval_body(self, env: Environment) -> None:
-        self.arg_list.eval(env=env)
+        self.argument_list.eval(env=env)
         values: List[Any] = env.stack.pop()
         env.stack.append(values)
