@@ -11,6 +11,7 @@ from moto.ds import ds_backends
 from moto.ds.models import Directory
 from moto.ec2 import ec2_backends
 from moto.moto_api._internal import mock_random
+from moto.utilities.utils import get_partition
 from moto.workspaces.exceptions import (
     InvalidParameterValuesException,
     ResourceAlreadyExistsException,
@@ -58,7 +59,8 @@ class Workspace(BaseModel):
         self.modification_states: List[
             Dict[str, str]
         ] = []  # modify_workspace_properties
-        self.related_workspaces: List[Dict[str, str]] = []  # create_standy_workspace
+        # create_standy_workspace
+        self.related_workspaces: List[Dict[str, str]] = []
         self.data_replication_settings: Dict[str, Any] = {}
         # The properties of the standby WorkSpace related to related_workspaces
         self.standby_workspaces_properties: List[Dict[str, Any]] = []
@@ -140,8 +142,16 @@ class WorkSpaceDirectory(BaseModel):
         self.subnet_ids = subnet_ids or dir_subnet_ids
         self.dns_ip_addresses = directory.dns_ip_addrs
         self.customer_username = "Administrator"
-        self.iam_rold_id = f"arn:aws:iam::{account_id}:role/workspaces_DefaultRole"
-        self.directory_type = directory.directory_type
+        self.iam_rold_id = (
+            f"arn:{get_partition(region)}:iam::{account_id}:role/workspaces_DefaultRole"
+        )
+        dir_type = directory.directory_type
+        if dir_type == "ADConnector":
+            self.directory_type = "AD_CONNECTOR"
+        elif dir_type == "SimpleAD":
+            self.directory_type = "SIMPLE_AD"
+        else:
+            self.directory_type = dir_type
         self.workspace_security_group_id = security_group_id
         self.state = "REGISTERED"
         # Default values for workspace_creation_properties
